use std::{io, path::Path, process::Command};

pub struct Build<'a> {
    dir: Option<&'a Path>,
    inputs: Vec<&'a str>,
}

impl<'a> Build<'a> {
    pub fn new() -> Self {
        Self {
            dir: None,
            inputs: Vec::new(),
        }
    }

    pub fn compile(self, output_path: &Path) -> io::Result<()> {
        let mut cmd = Command::new("cc");

        cmd.arg("-fpic").arg("-shared").arg("-std=c11");

        if let Some(path) = self.dir {
            cmd.current_dir(path);
        }

        cmd.args(&self.inputs);
        cmd.arg("-o").arg(output_path);

        dbg!(&cmd);

        let mut child = cmd.spawn()?;

        let _ = child.wait();

        Ok(())
    }

    pub fn dir(&mut self, path: &'a Path) -> &mut Self {
        self.dir = Some(path);
        self
    }

    pub fn input_files(&mut self, files: &[&'a str]) -> &mut Self {
        for file in files {
            self.inputs.push(*file);
        }
        self
    }
}
