use std::{
    env::consts::DLL_EXTENSION,
    fs::{canonicalize, create_dir_all},
    path::PathBuf,
};

use clap::Parser;
use git2::Repository;

mod cc;
mod languages;

use languages::{InstallInfo, get_install_info};
use settings::{Settings, get_settings};

#[derive(Debug, Parser)]
#[command(version, about, long_about = None)]
struct Cli {
    language: String,
}

fn main() -> anyhow::Result<()> {
    let Cli { language } = Cli::parse();

    let parser_dir = canonicalize(PathBuf::from("parsers"))?;
    let src_dir = PathBuf::from("parser_src");

    create_dir_all(&parser_dir)?;
    create_dir_all(&src_dir)?;

    let InstallInfo { url, files } =
        get_install_info(&language).ok_or(anyhow::anyhow!("unknown language `{language}`"))?;

    let repo_path = src_dir.join(&language);
    Repository::clone(url, &repo_path)?;

    let output_path = parser_dir.join(language).with_extension(DLL_EXTENSION);

    let mut build = cc::Build::new();
    build.dir(&repo_path).input_files(files);
    build.compile(&output_path)?;

    Ok(())
}
